package Tutorial2.Exercises_2;
import net.beadsproject.beads.data.Sample;
import net.beadsproject.beads.data.SampleManager;
import net.beadsproject.beads.ugens.*;
import net.happybrackets.core.HBAction;
import net.happybrackets.core.scheduling.Clock;
import net.happybrackets.device.HB;
import java.lang.invoke.MethodHandles;

/*
* Created by Garth Paine, June 2020
* This patch demonstrates the use of envelopes to change loop points
* and the use of the granular sample player to generate varying textures.
*
* An extension of this patch is to use the device ID to choose different samples on different devices
* thereby producing a spatial grain cloud
*
* Hack and enjoy
*
* nb. note that envelope segments are generated at compile time
* the for loop runs 50 times and generates lots of segments
* */

public class Extra_GrainsGalore implements HBAction {
    @Override
    public void action(HB hb) {

        //reset the device - stop previous sounding patch
        hb.reset();

    //create a gain object to control output gain
        Gain g = new Gain(2, 0.4f);
        hb.setStatus("LOADED Extra_GrainsGalore");

        //setup a delay buffer input and output
        TapIn ti = new TapIn( 4000);
        TapOut to = new TapOut( ti, 2400);
        //set the delay feedback amplitude ie. how much of the output gets fed back to the input
        Gain feedbackGain = new Gain( 1, 0.7f);
        //add the dely output to the feedbackGain
        feedbackGain.addInput(to);
        //add the feedback gain to the input of the delay to creat a feedback loop
        ti.addInput(feedbackGain);

        //same setup as above
        TapIn ti2 = new TapIn( 4800);
        TapOut to2 = new TapOut( ti2, 4800);
        Gain feedbackGain2 = new Gain( 2, 0.8f);
        feedbackGain2.addInput(to);
        ti2.addInput(feedbackGain2);

        //create a sample object to load the samples
        //Sample s = SampleManager.sample("data/audio/Drums/105a.wav");
        Sample s = SampleManager.sample("data/audio/Nylon_Guitar/Clean_D_harm.wav");
        Sample s2 = SampleManager.sample("data/audio/Nylon_Guitar/Clean_A_harm.wav");

        // an alternative with a drum sample
        //Sample s = SampleManager.sample("data/audio/Drums/02CountrySide.wav");


        //calculate the sample lenght so we know the parameters to change the loop points
        float sampleLength = (float) s.getLength();
        float sampleLength2 = (float) s2.getLength();

        hb.setStatus("Sample Length 1: " + sampleLength);

        //Create Granular Sample Players and set the loop type
        GranularSamplePlayer gsp = new GranularSamplePlayer( s);
        gsp.setLoopType(SamplePlayer.LoopType.LOOP_ALTERNATING);

        GranularSamplePlayer gsp2 = new GranularSamplePlayer( s2);
        gsp2.setLoopType(SamplePlayer.LoopType.LOOP_FORWARDS);


        //randomizes where in the file it looks for grains
        gsp.getRandomnessUGen().setValue(0.6f);
        gsp2.getRandomnessUGen().setValue(0.1f);

        //setting the grain size and interval
        gsp.getGrainIntervalUGen().setValue(60); //60, 120,   5
        gsp.getGrainSizeUGen().setValue(120); //30,  2,  120, 800

        gsp2.getGrainIntervalUGen().setValue(120); //60, 120,   5
        gsp2.getGrainSizeUGen().setValue(480); //30,  2,  120, 800

        //setting up envelopes for loopStart/End
        Envelope loopEnd = new Envelope( sampleLength);
        Envelope loopSetStart = new Envelope( 0.0f);

        Envelope loopEnd2 = new Envelope( sampleLength);
        Envelope loopSetStart2 = new Envelope( 0.0f);

        //setting up envelopes to slowly change the pitch and playback rate of the GranularSamplePlayers
        Envelope gspPitchEnv = new Envelope( 1f);
        Envelope gsp2RateEnv = new Envelope( 1f);

        Envelope grainSize = new Envelope( 60f);


        //float loopSetEnd = (sampleLength * ((random.nextFloat() + 1)/2.0f));

        //initial loopStart and loopEnd points
        float loopSetEnd = (sampleLength * ((hb.rng.nextFloat() + 1)/2.0f));
        float loopStart = (sampleLength * Math.abs(hb.rng.nextFloat())*0.5f);

        float loopSetEnd2 = (sampleLength2 * ((hb.rng.nextFloat() + 1)/2.0f));
        float loopStart2 = (sampleLength2 * Math.abs(hb.rng.nextFloat())*0.5f);

        //generate lots of segments so the envelopes play out for some time
        // note these are created at compile time and added together then, not during playback
        for (int i = 0; i<50; i++) {

            System.out.println(loopSetEnd + " loopSetEnd");
            loopEnd.addSegment(loopSetEnd, sampleLength);
            loopEnd.addSegment(loopSetEnd / 2, sampleLength);
            loopEnd.addSegment(loopSetEnd, sampleLength);
            loopEnd.addSegment(loopSetEnd / 16, sampleLength);
            loopEnd.addSegment(loopSetEnd, sampleLength);

            loopSetStart.addSegment(loopStart, sampleLength);
            loopSetStart.addSegment(0.25f, sampleLength);
            loopSetStart.addSegment(0.4f, sampleLength);
            loopSetStart.addSegment(0.125f, sampleLength);
            loopSetStart.addSegment(0.005f, sampleLength);
            loopSetStart.addSegment(0.25f, sampleLength);

            System.out.println(loopSetEnd2 + " loopSetEnd");
            loopEnd2.addSegment(loopSetEnd2, sampleLength2);
            loopEnd2.addSegment(loopSetEnd2 / 2, sampleLength2);
            loopEnd2.addSegment(loopSetEnd2, sampleLength2);
            loopEnd2.addSegment(loopSetEnd2 / 16, sampleLength2);
            loopEnd2.addSegment(loopSetEnd2, sampleLength2);

            loopSetStart2.addSegment(loopStart2, sampleLength2);
            loopSetStart2.addSegment(0.25f, sampleLength2);
            loopSetStart2.addSegment(0.4f, sampleLength2);
            loopSetStart2.addSegment(0.125f, sampleLength2);
            loopSetStart2.addSegment(0.005f, sampleLength2);
            loopSetStart2.addSegment(0.25f, sampleLength2);


            gspPitchEnv.addSegment(1.2f, sampleLength*4);
            gspPitchEnv.addSegment(0.8f, sampleLength*4);

            gsp2RateEnv.addSegment(0.25f, sampleLength2*8);
            gsp2RateEnv.addSegment(0.5f, sampleLength2*8);
            gsp2RateEnv.addSegment(1f, sampleLength2*8);


            grainSize.addSegment(800, sampleLength*4);
            grainSize.addSegment(60, sampleLength*4);
            grainSize.addSegment(10, sampleLength);
            grainSize.addSegment(30, sampleLength);


        }

        //changes the loopStart to avoid the attack on the string
        gsp.setLoopStart(loopSetStart);
        gsp2.setLoopStart(loopSetStart2);

        //set the loopEnd points using the envelope
        gsp.setLoopEnd(loopEnd);
        gsp2.setLoopEnd(loopEnd2);

        //randomly scatter the grains in the stereo field
        gsp.getRandomPanUGen().setValue(0.2f);
        gsp2.getRandomPanUGen().setValue(0.7f);


        //change the grain size
        gsp.setGrainSize(grainSize);


        gsp.setPitch(gspPitchEnv);
        gsp2.setRate(gsp2RateEnv);

        //plug the Delays in and add to the output
        ti.addInput(gsp);
        ti2.addInput(gsp2);
        //add the tap out delays to the gain object
        g.addInput(to);
        g.addInput(to2);
        //add the granular sample players to the gain object
        g.addInput(gsp);
        g.addInput(gsp2);
        //send the gain object out
        hb.sound(g);


        // I use this clock messageListener to print out information occassionally

        // To create this, just type clockTimer
        Clock clock = HB.createClock(500).addClockTickListener((offset, this_clock) -> {// Write your code below this line

            //you can use this to look at the rate of the ticks being produced by the clock
            //hb.setStatus("ticks: " + this_clock.getNumberTicks());

            //remember you can not run too many grains on a Pi Zero so this helps to check how many can be run before overload
            if (this_clock.getNumberTicks()%10 == 0) {
                hb.setStatus("grains gsp: " + gsp.getAverageNumberOfGrains());
            }

            if (this_clock.getNumberTicks()%15 == 0) {
                hb.setStatus("grains gsp2: " + gsp2.getAverageNumberOfGrains());
            }
            // Write your code above this line
        });

        clock.start();// End Clock Timer

        //this codes gets the index of the device and presents it as a positive number
        //you could use this ie. myIndex%2==0 to test if the device id is Odd or Even and play different samples accordingly
        int myIndex = Math.abs(hb.myIndex());
        hb.setStatus("My Index is: " + Math.abs(myIndex));



    }


    //<editor-fold defaultstate="collapsed" desc="Debug Start">

    /**
     * This function is used when running sketch in IntelliJ IDE for debugging or testing
     *
     * @param args standard args required
     */
    public static void main(String[] args) {

        try {
            HB.runDebug(MethodHandles.lookup().lookupClass());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    //</editor-fold>
}
