package Tutorial2.Exercises_2;

import net.beadsproject.beads.data.Buffer;
import net.beadsproject.beads.data.Pitch;
import net.beadsproject.beads.data.Sample;
import net.beadsproject.beads.data.SampleManager;
import net.beadsproject.beads.events.KillTrigger;
import net.beadsproject.beads.ugens.*;
import net.happybrackets.core.HBAction;
import net.happybrackets.core.scheduling.Clock;
import net.happybrackets.device.HB;

import java.lang.invoke.MethodHandles;

/*
 * Created by Garth Paine, June 2020
 * This patch demonstrates the use of both a sample player with dynamic pitch and grain size variables
 * and the simultaneous performance of a random melody using a WavePlayer
 *
 * Note the use of the hb.rng.nextFloat() to generate a random number for each melody note
 * Also not the use of the Pitch class to constrain the bass line to a minor scale
 *
 * An extension of this patch would be to explore the Pitch class to constrain the notes to a scale
 *
 * Hack and enjoy
 *
 * nb. note that envelope segments are generated at compile time
 * the for loop runs 50 times and generates lots of segments
 *
 * nbb. Also note that hb.sound(gWp); controls the number of voices playing simultaneously
 * this is called voice stealing and when the 5th voice is created the oldest one is killed
 * */

public class Extra_Melody_Granular implements HBAction {
    @Override
    public void action(HB hb) {
        hb.reset(); //Clears any running code on the device
        //generate a sample manager - in this case a group of samples in the folder Drums
        SampleManager.group("Drums", "data/audio/Mattel_Drum_Machine");


        // To create this, just type clockTimer
        Clock clock = HB.createClock(500).addClockTickListener((offset, this_clock) -> {// Write your code below this line

            //use the clock to trigger sample playback on each tick
            Sample s = SampleManager.randomFromGroup("Drums");

            GranularSamplePlayer gsp = new GranularSamplePlayer(s);

            //note the use of the alternating loop and hear the interesting variations this makes
            gsp.setLoopType(GranularSamplePlayer.LoopType.LOOP_ALTERNATING);

            //setting the grain size and interval
            gsp.getGrainIntervalUGen().setValue(120); //60, 120,   5
            gsp.getGrainSizeUGen().setValue(800); //30,  2,  120, 800

            Envelope ePitch = new Envelope(1f);
            ePitch.addSegment(4f, 2000);
            ePitch.addSegment(0.5f, 4000);

            //gsp.getPitchUGen().setValue(4f);
            gsp.setPitch(ePitch);

            //use a gain object to control the output amplitude
            Gain g = new Gain(1, 0.9f);

            //add the granularSamplePlayer to the gain object
            g.addInput(gsp);

            //connect the gain object to the audio hardware output
            hb.sound(g);

                //show the tick count in the status bar
                hb.setStatus("ticks: " + this_clock.getNumberTicks());

                //create a WavePlayer and ADSR amplitude envelop for the bass melody note
//                if (this_clock.getNumberTicks() % 4 == 0){
//                    int pitch = Pitch.forceToScale(Math.abs(hb.rng.nextInt(16)) + 24, Pitch.minor);
//                    //and then convert it to frequency as the WavePlayer object requires a frequency
//                    float freq = Pitch.mtof(pitch);
//
//                    WavePlayer wp = new WavePlayer(freq, Buffer.SAW);
//                    Envelope eWaveAmp = new Envelope(0f);
//                    eWaveAmp.addSegment(0.4f, 100);
//                    eWaveAmp.addSegment(0.25f, 10);
//                    eWaveAmp.addSegment(0.1f, 1000);
//
//
//                    //note this envelop should include a KillTrigger to dispose of the WavePlayer when the note is finished
//                    //see if you can work out how to add that here
//
//                    Gain gWp = new Gain(1, eWaveAmp);
//                    eWaveAmp.addSegment(0f, 300, new KillTrigger(gWp));
//
//                    gWp.addInput(wp);
//
//                    hb.sound(gWp);
//            }

            // Write your code above this line
        });

        clock.start();// End Clock Timer

        // write your code above this line
    }


    //<editor-fold defaultstate="collapsed" desc="Debug Start">

    /**
     * This function is used when running sketch in IntelliJ IDE for debugging or testing
     *
     * @param args standard args required
     */
    public static void main(String[] args) {

        try {
            HB.runDebug(MethodHandles.lookup().lookupClass());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    //</editor-fold>
}
