package Tutorial2;

import net.beadsproject.beads.data.Sample;
import net.beadsproject.beads.data.SampleManager;
import net.beadsproject.beads.ugens.Envelope;
import net.beadsproject.beads.ugens.Gain;
import net.beadsproject.beads.ugens.GranularSamplePlayer;
import net.happybrackets.core.HBAction;
import net.happybrackets.core.scheduling.Clock;
import net.happybrackets.device.HB;

import java.lang.invoke.MethodHandles;

public class GrainDrums implements HBAction {
    @Override
    public void action(HB hb) {
        hb.reset(); //Clears any running code on the device
        SampleManager.group("Drums", "data/audio/Mattel_Drum_Machine");


        // To create this, just type clockTimer
        Clock clock = HB.createClock(500).addClockTickListener((offset, this_clock) -> {// Write your code below this line

            //use the clock to trigger sample playback on each tick
            Sample s = SampleManager.randomFromGroup("Drums");

            GranularSamplePlayer gsp = new GranularSamplePlayer(s);

            //note the use of the alternating loop and hear the interesting variations this makes
            gsp.setLoopType(GranularSamplePlayer.LoopType.LOOP_ALTERNATING);

            //setting the grain size and interval
            gsp.getGrainIntervalUGen().setValue(120); //60, 120,   5
            gsp.getGrainSizeUGen().setValue(800); //30,  2,  120, 800

            Envelope ePitch = new Envelope(1f);
            ePitch.addSegment(4f, 2000);
            ePitch.addSegment(0.5f, 4000);

            //gsp.getPitchUGen().setValue(4f);
            gsp.setPitch(ePitch);

            //use a gain object to control the output amplitude
            Gain g = new Gain(1, 0.9f);

            //add the granularSamplePlayer to the gain object
            g.addInput(gsp);

            //connect the gain object to the audio hardware output
            hb.sound(g);

            //show the tick count in the status bar
            hb.setStatus("ticks: " + this_clock.getNumberTicks());



            // Write your code above this line
        });

        clock.start();// End Clock Timer
    }


    //<editor-fold defaultstate="collapsed" desc="Debug Start">

    /**
     * This function is used when running sketch in IntelliJ IDE for debugging or testing
     *
     * @param args standard args required
     */
    public static void main(String[] args) {

        try {
            HB.runDebug(MethodHandles.lookup().lookupClass());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    //</editor-fold>
}
