package Tutorial2;

import net.beadsproject.beads.data.Sample;
import net.beadsproject.beads.data.SampleManager;
import net.beadsproject.beads.events.KillTrigger;
import net.beadsproject.beads.ugens.Envelope;
import net.beadsproject.beads.ugens.Gain;
import net.beadsproject.beads.ugens.SamplePlayer;
import net.happybrackets.core.HBAction;
import net.happybrackets.core.scheduling.Clock;
import net.happybrackets.device.HB;

import java.lang.invoke.MethodHandles;

public class SimpleSamplePlayer implements HBAction {
    @Override
    public void action(HB hb) {
        hb.reset(); //Clears any running code on the device

        //Sample sample = SampleManager.sample("data/audio/Nylon_Guitar/Clean_A_harm.wav");
        //Sample sample = SampleManager.sample("data/audio/Drums/EH_DrumLoop130_1.wav");
        Sample glitchSample = SampleManager.sample("data/audio/long/DrumLoop130.wav");
        Sample baseSample = SampleManager.sample("data/audio/long/1979.wav");

/*
        //This should play as the steady loop
        SamplePlayer sp = new SamplePlayer(baseSample);
        sp.setLoopType(SamplePlayer.LoopType.LOOP_FORWARDS);
        // Gain object to control the output gain
        Gain gainSP = new Gain( 2, 0.8f);
        gainSP.addInput(sp);
        hb.sound(gainSP);
*/


        // To create this, just type clockTimer
        //this clock triggers an action every 500ms = 120BPM, so 250ms = 60BPM
        Clock clock = HB.createClock(250).addClockTickListener((offset, this_clock) -> {// Write your code below this line

            hb.setStatus("tick count: " + this_clock.getNumberTicks());

                        if ((this_clock.getNumberTicks()%8) == 0){

                            hb.setStatus("Glitch Sample");

                            SamplePlayer spGlitch = new SamplePlayer( glitchSample);
                            float sampleLength = (float) glitchSample.getLength();

                            // Gain object to control the output gain
                            Gain g = new Gain( 2, 0.3f);

                            // envelope to control rate of sample player
                            Envelope pitchEnvelope = new Envelope( 1); //normal playback rate
                            //add segment to the pitch envelope - set the pitch (rate of playback) randomly between -2 and 2
                            // try changing the range of speed changes and the rate at which the speed is set
                            //pitchEnvelope.addSegment((float) (2f*Math.abs(r.nextFloat())), 1000);
                            pitchEnvelope.addSegment((float) (0.25 + (2f*(Math.abs(hb.rng.nextFloat())))), 250*Math.abs(hb.rng.nextFloat()));

                            //we then use the pitchEnvelop to set the playback rate of the sample
                            spGlitch.setRate(pitchEnvelope);

                            //the loop type is set
                            //sp.setLoopType(SamplePlayer.LoopType.LOOP_ALTERNATING);
                            spGlitch.setLoopType(SamplePlayer.LoopType.LOOP_FORWARDS);
                            Envelope loopEnd = new Envelope( 1000);
                            Envelope loopStart = new Envelope( 0f);
                            spGlitch.setLoopStart(loopStart);
                            spGlitch.setLoopEnd(loopEnd);

                            //the loopStart and LoopEnd points can be set - here th LoopStart is randmonly set
                            loopStart.addSegment((float) (0.2 * (Math.abs(hb.rng.nextFloat()))), 10);

                            // here the loopEnd point is set to half the sample length =  random amount of the remaining half
                            loopEnd.addSegment((float) ((sampleLength*0.2f)+(0.5*(Math.abs(hb.rng.nextFloat())))), 100);
                            loopEnd.addSegment(sampleLength, 2000f, new KillTrigger(g));

                            g.addInput(spGlitch);

                            // this has voice stealing - limits voices to 4
                            hb.sound(g);

                        }

                            if ((this_clock.getNumberTicks()%32) == 0) {

                                //hb.setStatus(String.valueOf(hb.clock.getCount()));

                                SamplePlayer spBass = new SamplePlayer(baseSample);
                                float sampleLength = (float) baseSample.getLength();
                                // Gain object to control the output gain
                                Gain g = new Gain(2, 0.3f);

                                // envelope to control rate of sample player
                                Envelope pitchEnvelope = new Envelope(1); //normal playback rate
                                //add segment to the pitch envelope - set the pitch (rate of playback) randomly between -2 and 2
                                // try changing the range of speed changes and the rate at which the speed is set
                                //pitchEnvelope.addSegment((float) (2f*Math.abs(r.nextFloat())), 1000);
                                pitchEnvelope.addSegment((float) (2f * (hb.rng.nextFloat())), 250 * Math.abs(hb.rng.nextFloat()));

                                //we then use the pitchEnvelop to set the playback rate of the sample
                                spBass.setRate(pitchEnvelope);

                                //the loop type is set
                                //sp.setLoopType(SamplePlayer.LoopType.LOOP_ALTERNATING);
                                spBass.setLoopType(SamplePlayer.LoopType.LOOP_FORWARDS);
                                Envelope loopEnd = new Envelope(1000);
                                Envelope loopStart = new Envelope(0f);
                                spBass.setLoopStart(loopStart);
                                spBass.setLoopEnd(loopEnd);

                                //the loopStart and LoopEnd points can be set - here th LoopStart is randmonly set
                                loopStart.addSegment((float) (0.2 * (Math.abs(hb.rng.nextFloat()))), 10);

                                // here the loopEnd point is set to half the sample length =  random amount of the remaining half
                                loopEnd.addSegment((float) ((sampleLength * 0.2f) + (0.5 * (Math.abs(hb.rng.nextFloat())))), 100);
                                loopEnd.addSegment(sampleLength, 2000f, new KillTrigger(g));

                                g.addInput(spBass);

                                // this has voice stealing - limits voices to 4
                                hb.sound(g);

                        };




            // Write your code above this line
        });


        clock.start();/* End Clock Timer */


    }


    //<editor-fold defaultstate="collapsed" desc="Debug Start">

    /**
     * This function is used when running sketch in IntelliJ IDE for debugging or testing
     *
     * @param args standard args required
     */
    public static void main(String[] args) {

        try {
            HB.runDebug(MethodHandles.lookup().lookupClass());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    //</editor-fold>
}
