package Tutorial2;

import net.beadsproject.beads.data.Buffer;
import net.beadsproject.beads.ugens.Envelope;
import net.beadsproject.beads.ugens.Gain;
import net.beadsproject.beads.ugens.WavePlayer;
import net.happybrackets.core.HBAction;
import net.happybrackets.device.HB;

import java.lang.invoke.MethodHandles;

/*
 * Created by Garth Paine, June 2020
 * This patch demonstrates the extension of the HelloSound patch to use enveloped
 * to change the amplitude of the output and
 * to change the pitch of the oscillator
 *
 * here the oscillator is always playing even though the amplitude is turned down to zero
 *
 * Later we will lear about the KillTrigger which will allow us to remove the oscilator when we have finished with it to save processing cycles
 *
 * */

public class SoundOne implements HBAction {
    @Override
    public void action(HB hb) {
        hb.reset(); //Clears any running code on the device
        //Write your sketch below

        //envelop to control gain of output
        Envelope gainEnv = new Envelope(0f);
        gainEnv.addSegment(0.5f, 10 );
        gainEnv.addSegment(0f, 2000);
        //add two more segments to form an ADSR envelop

        //envelop to control pitch of playback
        Envelope pitchEnv = new Envelope(50f);
        pitchEnv.addSegment(400, 1000);
        pitchEnv.addSegment(100, 500);

        //waveplayer to synthesis sound
        WavePlayer wp = new WavePlayer(pitchEnv, Buffer.SAW);
        //Gain object to control output volume
        Gain g = new Gain(1, 0.2f);
        //plug waveplayer into gain input
        g.addInput(wp);
        //connect gain object to sound output hardware
        hb.sound(g);


        // write your code above this line
    }


    //<editor-fold defaultstate="collapsed" desc="Debug Start">

    /**
     * This function is used when running sketch in IntelliJ IDE for debugging or testing
     *
     * @param args standard args required
     */
    public static void main(String[] args) {

        try {
            HB.runDebug(MethodHandles.lookup().lookupClass());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    //</editor-fold>
}
